//
// Copyright (c) Microsoft Corporation.  All rights reserved.
//
//
// Use of this source code is subject to the terms of the Microsoft end-user
// license agreement (EULA) under which you licensed this SOFTWARE PRODUCT.
// If you did not accept the terms of the EULA, you are not authorized to use
// this source code. For a copy of the EULA, please see the LICENSE.RTF on your
// install media.
//
/*++

THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF
ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
PARTICULAR PURPOSE.

Module Name:
    EAU.c

Abstract:
   EAU_Xxx Streams interface for USB  Client Driver.

Functions:

Notes:

--*/


#include "ECAIOUSB.h"
#include "..\inc\EAU.h"



BOOL
RegisterECAIOUSBSettings(
   PECAIOUSB_CONTEXT pContext
   );

BOOL
DeregisterECAIOUSBSettings(
   PECAIOUSB_CONTEXT pContext
   );

PECAIOUSB_CONTEXT
GetContextFromReg(
   LPTSTR  ActivePath
   );

DWORD
EAU_ReadComplete(
   PVOID    Context
   );

DWORD
EAU_WriteComplete(
   PVOID    Context
   );

DWORD GenericVendorRead(PECAIOUSB_CONTEXT pContext, BYTE Request, USHORT Value, USHORT Index, PDWORD pDataSize, PVOID pData);
DWORD GenericVendorWrite(PECAIOUSB_CONTEXT pContext, BYTE Request, USHORT Value, USHORT Index, PDWORD pDataSize, PVOID pData);


/*++
Called by Device Manager to initialize the streams interface in response to ActivateDevice.
We passed ActivateDevice a pointer to our device context, but must read it out of the registry as "ClientInfo".

Returns context used in XXX_Open, XXX_PowerDown, XXX_PowerUp, and XXX_Deinit
--*/
PECAIOUSB_CONTEXT
EAU_Init(
   PVOID Context
   )
{
   LPTSTR ActivePath = (LPTSTR)Context; // HKLM\Drivers\Active\xx
   PECAIOUSB_CONTEXT pContext = NULL;
   BOOL bRc = FALSE;

   DEBUGMSG(ZONE_EAU_INIT, (TEXT(">EAU_Init(%p)\n"), Context));

   //
   // get our Context
   //
   pContext = GetContextFromReg( ActivePath );

   //
   // Register our file device (EAU) name
   //
   if ( VALID_CONTEXT( pContext ) ) {

      EnterCriticalSection(&pContext->Lock);

      do {

         if ( pContext->ActivePath ) {
            DEBUGMSG(ZONE_ERR, (TEXT("Existing ActivePath: %s\n"), pContext->ActivePath));
            TEST_TRAP();
            break;
         }

         pContext->ActivePath = ActivePath;


         bRc = RegisterECAIOUSBSettings( pContext );

         if ( !bRc ) {
            DEBUGMSG(ZONE_ERR, (TEXT("RegisterECAIOUSBSettings Failed\n")));
            break;
         }

      } while (0);

      LeaveCriticalSection(&pContext->Lock);
   }

   DEBUGMSG(ZONE_EAU_INIT, (TEXT("<EAU_Init:0x%x\n"), pContext ));

   return (bRc ? pContext : NULL);
}


BOOL
EAU_Deinit(
   PECAIOUSB_CONTEXT pContext
   )
{
   BOOL  bRc = FALSE;

   DEBUGMSG(ZONE_EAU_INIT, (TEXT(">EAU_Deinit\n")));

   if ( VALID_CONTEXT( pContext ) ) {

      EnterCriticalSection( &pContext->Lock );

#if DEBUG
      if (pContext->Flags.Open) {
         DEBUGMSG(ZONE_ERR, (TEXT("EAU_Deinit on Open Device!\n")));
         TEST_TRAP();
      }
#endif

      bRc = DeregisterECAIOUSBSettings( pContext );
      if ( !bRc ) {
         DEBUGMSG(ZONE_ERR, (TEXT("DeregisterECAIOUSBSettings Failed\n")));
      }

      LeaveCriticalSection(&pContext->Lock);

   }

   DEBUGMSG(ZONE_EAU_INIT, (TEXT("<EAU_Deinit:%d\n"), bRc));

   return bRc;
}


//
// Returns open context to be used in the
// XXX_Read, XXX_Write, XXX_Seek, and XXX_IOControl functions.
// If the device cannot be opened, this function returns NULL.
//
PECAIOUSB_CONTEXT
EAU_Open(
   PECAIOUSB_CONTEXT Context,      // context returned by EAU_Init.
   DWORD           AccessCode,   // @parm access code
   DWORD           ShareMode     // @parm share mode
   )
{
   PECAIOUSB_CONTEXT pContext = Context;
   BOOL bRc = TRUE;

   UNREFERENCED_PARAMETER(ShareMode);
   UNREFERENCED_PARAMETER(AccessCode);

   DEBUGMSG(ZONE_EAU_INIT,(TEXT(">EAU_Open(0x%x, 0x%x, 0x%x)\n"),pContext, AccessCode, ShareMode));

   if ( VALID_CONTEXT( pContext ) ) {

      EnterCriticalSection(&pContext->Lock);

        if ( !pContext->Flags.Open &&
           !pContext->Flags.UnloadPending ) {

            pContext->Flags.Open = TRUE;

         ResetEvent( pContext->hCloseEvent ); // non-signaled

        } else {
            DEBUGMSG( ZONE_ERR,(TEXT("EAU_Open: ERROR_ACCESS_DENIED\n")));
            SetLastError(ERROR_ACCESS_DENIED);
            bRc = FALSE;
        }

      LeaveCriticalSection(&pContext->Lock);

   } else {
      DEBUGMSG( ZONE_ERR,(TEXT("EAU_Open: ERROR_FILE_NOT_FOUND\n")));
        SetLastError(ERROR_FILE_NOT_FOUND);
      bRc = FALSE;
    }

   DEBUGMSG(ZONE_EAU_INIT,(TEXT("<EAU_Open:%d\n"), bRc ));

   return (bRc ? pContext : NULL);
}


BOOL
EAU_Close(
   PECAIOUSB_CONTEXT Context
   )
{
   PECAIOUSB_CONTEXT pContext = Context;

   DEBUGMSG(ZONE_EAU_INIT,(TEXT("EAU_Close(0x%x)\n"),pContext));

   if ( VALID_CONTEXT( pContext ) ) {

      EnterCriticalSection(&pContext->Lock);

      pContext->Flags.Open = FALSE;

      LeaveCriticalSection(&pContext->Lock);

      //
      // Note: any waiters are run as soon as we signal this event
      //
      return SetEvent( pContext->hCloseEvent );

   } else {
      DEBUGMSG( ZONE_ERR,(TEXT("EAU_Close: ERROR_INVALID_HANDLE\n")));
        SetLastError(ERROR_INVALID_HANDLE);
      return FALSE;
   }
}

#define MAX_USBTRANSFER_SIZE 0x1000 // 4k
//
// Printers are sequential byte processing devices,
// so no need for anything fancy
//
ULONG
EAU_Write(
   PECAIOUSB_CONTEXT pContext,
   PUCHAR pBuffer,
   ULONG  BufferLength
   )
{
   DWORD dwTimeout;
   DWORD dwTransferSize;
   DWORD dwBytesTransferred ;
   DWORD dwTatalTransferred = 0 ;
   DWORD dwErr = ERROR_SUCCESS;
   DWORD dwUsbErr = USB_NO_ERROR;

   DEBUGMSG(ZONE_EAU_WRITE,(TEXT(">EAU_Write(0x%x, %d)\n"),pBuffer, BufferLength));

   if ( ACCEPT_IO( pContext ) ) {

      while  ( pBuffer && BufferLength) {

         dwTransferSize=min(BufferLength,MAX_USBTRANSFER_SIZE);
         dwTimeout = pContext->UsbTimeouts.WriteTimeout;

         if (!dwTimeout) {
            dwTimeout = INFINITE;
         }

         DEBUGMSG(ZONE_EAU_WRITE, (TEXT("EAU_Write timeout due in %d msec\n"), dwTimeout));
         EnterCriticalSection( &pContext->Lock );
         dwBytesTransferred = 0 ;
         dwErr = IssueBulkTransfer( pContext->UsbFuncs,
                                    pContext->BulkOut.hPipe,
                                    DefaultTransferComplete,  // Callback
                                    pContext->BulkOut.hEvent,  // Context
                                    (USB_OUT_TRANSFER /*| USB_SHORT_TRANSFER_OK*/), // Flags
                                    pBuffer, 0,
                                    dwTransferSize,
                                   &dwBytesTransferred,
                                    dwTimeout,
                                   &dwUsbErr );

         if ( ERROR_SUCCESS != dwErr || USB_NO_ERROR != dwUsbErr) {
            DEBUGMSG( ZONE_ERR, (TEXT("IssueBulkTransfer error:%d, 0x%x\n"), dwErr, dwUsbErr));
            IoErrorHandler( pContext, pContext->BulkOut.hPipe, pContext->BulkOut.bIndex, dwUsbErr);
            LeaveCriticalSection( &pContext->Lock );
            break;
         }
         else {
            LeaveCriticalSection( &pContext->Lock );
            pBuffer +=dwTransferSize;
            BufferLength -=dwTransferSize;
            dwTatalTransferred +=dwTransferSize;
         }

      };
   } else {
      DEBUGMSG( ZONE_ERR,(TEXT("EAU_Write: ERROR_INVALID_HANDLE\n")));
      dwTatalTransferred=(DWORD)-1;
      SetLastError(ERROR_INVALID_HANDLE);
    }

   DEBUGMSG(ZONE_EAU_WRITE,(TEXT("<EAU_Write:%d\n"), dwTatalTransferred ));

   return dwTatalTransferred;
}


ULONG
EAU_Read(
   PECAIOUSB_CONTEXT pContext,
   PUCHAR pBuffer,
   ULONG  BufferLength
   )
{
   DWORD dwTimeout;
   DWORD dwBytesTransferred = 0;
   DWORD dwErr = ERROR_SUCCESS;
   DWORD dwUsbErr = USB_NO_ERROR;

   DEBUGMSG(ZONE_EAU_READ,(TEXT(">EAU_Read(0x%x, %d)\n"),pBuffer, BufferLength));

   if ( ACCEPT_IO( pContext ) ) {

      if ( pBuffer && BufferLength ) {

         EnterCriticalSection( &pContext->Lock );

         dwTimeout = pContext->UsbTimeouts.ReadTimeout;

         if (!dwTimeout) {
            dwTimeout = INFINITE;
         }

         DEBUGMSG(ZONE_EAU_READ, (TEXT("EAU_Read timeout due in %d msec\n"), dwTimeout));

         dwErr = IssueBulkTransfer( pContext->UsbFuncs,
                                    pContext->BulkIn.hPipe,
                                    DefaultTransferComplete,    // Callback
                                    pContext->BulkIn.hEvent,     // Context
                                    (USB_IN_TRANSFER | USB_SHORT_TRANSFER_OK), // Flags
                                    pBuffer, 0,
                                    BufferLength,
                                   &dwBytesTransferred,
                                    dwTimeout,
                                   &dwUsbErr );

         if ( ERROR_SUCCESS != dwErr || USB_NO_ERROR != dwUsbErr) {
            DEBUGMSG( ZONE_ERR, (TEXT("IssueBulkTransfer error:%d, 0x%x\n"), dwErr, dwUsbErr));
            IoErrorHandler( pContext, pContext->BulkOut.hPipe, pContext->BulkOut.bIndex, dwUsbErr);
         }

         LeaveCriticalSection( &pContext->Lock );

      } else {
         DEBUGMSG( ZONE_ERR,(TEXT("EAU_Read: ERROR_INVALID_PARAMETER\n")));
           SetLastError(ERROR_INVALID_PARAMETER );
       }


   } else {
      DEBUGMSG( ZONE_ERR,(TEXT("EAU_Read: ERROR_INVALID_HANDLE\n")));
        SetLastError(ERROR_INVALID_HANDLE);
    }

   DEBUGMSG(ZONE_EAU_READ,(TEXT("<EAU_Read:%d\n"), dwBytesTransferred));

   return dwBytesTransferred;
}



BOOL
EAU_IOControl(
    PECAIOUSB_CONTEXT pContext,
    DWORD dwCode,
    PBYTE pBufIn,
    DWORD dwLenIn,
    PBYTE pBufOut,
    DWORD dwLenOut,
    PDWORD pdwActualOut
   )
{
	BOOL bRc = FALSE;
	PVENDOR_REQUEST_DATA pVRD;
	DWORD BytesWritten, IOStatus;

	DEBUGMSG(ZONE_EAU_IOCTL,(TEXT(">EAU_IOControl(0x%x, 0x%x, %d, 0x%x)\n"), dwCode, pBufIn, dwLenIn, pBufOut, dwLenOut ));

	if ( ACCEPT_IO( pContext ) )
	{

		switch ( dwCode )
		{
			case IOCTL_EAU_WRITE:
				DEBUGMSG(ZONE_EAU_IOCTL, (TEXT("IOCTL_EAU_WRITE\n")));
				if ( dwLenIn < sizeof(VENDOR_REQUEST_DATA) )
				{
					DEBUGMSG(ZONE_ERR, (TEXT("EAU_IOControl: ERROR_INSUFFICIENT_BUFFER\n")));
					SetLastError(ERROR_INSUFFICIENT_BUFFER);
					bRc = FALSE;
					break;
				}
				if ( dwLenIn > sizeof(VENDOR_REQUEST_DATA) + 255 )
				{
					DEBUGMSG(ZONE_ERR, (TEXT("EAU_IOControl: ERROR_BUFFER_OVERFLOW\n")));
					SetLastError(ERROR_BUFFER_OVERFLOW);
					bRc = FALSE;
					break;
				}
				EnterCriticalSection(&pContext->Lock);
				//CeSafeCopyMemory(&pContext->Timeouts, pBufIn, sizeof(pContext->Timeouts));
				pVRD = (PVENDOR_REQUEST_DATA)pBufIn;
				BytesWritten = dwLenIn - sizeof(VENDOR_REQUEST_DATA);
				IOStatus = GenericVendorWrite(pContext, pVRD->Request, pVRD->Value, pVRD->Index, &BytesWritten, pBufIn + sizeof(VENDOR_REQUEST_DATA));
				LeaveCriticalSection(&pContext->Lock);

				*pdwActualOut = 1;
				if ( IOStatus == ERROR_SUCCESS )
				{
					*pBufOut = (BYTE)BytesWritten;
					bRc = TRUE;
				}
				else
				{
					*pBufOut = 0;
					bRc = FALSE;
				}
				SetLastError(IOStatus);
			break;

			case IOCTL_EAU_READ:
				DEBUGMSG(ZONE_EAU_IOCTL, (TEXT("IOCTL_EAU_READ\n")));
				if ( (dwLenIn < sizeof(VENDOR_REQUEST_DATA)) || (dwLenOut < 1) )
				{
					DEBUGMSG(ZONE_ERR, (TEXT("EAU_IOControl: ERROR_INSUFFICIENT_BUFFER\n")));
					SetLastError(ERROR_INSUFFICIENT_BUFFER);
					bRc = FALSE;
					break;
				}
				if ( dwLenOut > 255 )
				{
					DEBUGMSG(ZONE_ERR, (TEXT("EAU_IOControl: ERROR_BUFFER_OVERFLOW\n")));
					SetLastError(ERROR_BUFFER_OVERFLOW);
					bRc = FALSE;
					break;
				}
				EnterCriticalSection(&pContext->Lock);
				pVRD = (PVENDOR_REQUEST_DATA)pBufIn;
				*pdwActualOut = dwLenOut;
				IOStatus = GenericVendorRead(pContext, pVRD->Request, pVRD->Value, pVRD->Index, pdwActualOut, pBufOut);
				LeaveCriticalSection(&pContext->Lock);

				if ( IOStatus == ERROR_SUCCESS )
				{
					bRc = TRUE;
				}
				else
				{
					*pdwActualOut = 0;
					bRc = FALSE;
				}
				SetLastError(IOStatus);
			break;

			default:
				DEBUGMSG( ZONE_ERR,(TEXT("EAU_IOControl(0x%x) : ERROR_NOT_SUPPORTED\n"), dwCode));
				SetLastError(ERROR_NOT_SUPPORTED);
				bRc = FALSE;
			break;
		}

	}
	else
	{
		DEBUGMSG( ZONE_ERR,(TEXT("EAU_IOControl: ERROR_INVALID_HANDLE\n")));
		SetLastError(ERROR_INVALID_HANDLE);
		bRc = FALSE;
	}

	DEBUGMSG(ZONE_EAU_IOCTL,(TEXT("<EAU_IOControl:%d\n"), bRc));

	return bRc;
}

DWORD GenericVendorRead(PECAIOUSB_CONTEXT pContext, BYTE Request, USHORT Value, USHORT Index, PDWORD pDataSize, PVOID pData)
{
	USB_DEVICE_REQUEST ControlHeader;
	DWORD dwBytesTransferred = 0;
	DWORD dwUsbErr = USB_NO_ERROR;
	DWORD dwErr = ERROR_SUCCESS;
	DWORD usStatus = 0;
	DWORD Result = ERROR_SUCCESS;

	DEBUGMSG( ZONE_USB_CONTROL, (TEXT(">ECAIOUSB_GenericVendorRead\n")));

	if ( ! VALID_CONTEXT(pContext) )
	{
		DEBUGMSG( ZONE_ERR, (TEXT("Invalid parameter\n")));
		Result = ERROR_INVALID_PARAMETER;
	}
	else
	{
		ControlHeader.bmRequestType = USB_REQUEST_DEVICE_TO_HOST | USB_REQUEST_VENDOR | USB_REQUEST_FOR_DEVICE;
		ControlHeader.bRequest = Request;
		ControlHeader.wValue = Value;
		ControlHeader.wIndex = Index;
		ControlHeader.wLength = (USHORT)*pDataSize;

		dwErr = IssueVendorTransfer(pContext->UsbFuncs, pContext->hUsbDevice, DefaultTransferComplete, pContext->hEP0Event, (USB_IN_TRANSFER | USB_SHORT_TRANSFER_OK), &ControlHeader, pData, 0, &dwBytesTransferred, pContext->UsbTimeouts.ReadTimeout, &dwUsbErr);
		*pDataSize = dwBytesTransferred;

		if ( (dwErr != ERROR_SUCCESS) || (dwUsbErr != USB_NO_ERROR) )
		{
			DEBUGMSG( ZONE_ERR, (TEXT("IssueVendorTransfer ERROR:%d 0x%x\n"), dwErr, dwUsbErr));
			IoErrorHandler(pContext, NULL, 0, dwUsbErr);
			if ( dwErr == ERROR_SUCCESS ) Result = ERROR_GEN_FAILURE; else Result = dwErr;
		}
	}

	DEBUGMSG( ZONE_USB_CONTROL, (TEXT("<ECAIOUSB_GenericVendorRead: 0x%x\n"), usStatus));

	return Result;
}

DWORD GenericVendorWrite(PECAIOUSB_CONTEXT pContext, BYTE Request, USHORT Value, USHORT Index, PDWORD pDataSize, PVOID pData)
{
	USB_DEVICE_REQUEST ControlHeader;
	DWORD dwBytesTransferred = 0;
	DWORD dwUsbErr = USB_NO_ERROR;
	DWORD dwErr = ERROR_SUCCESS;
	DWORD Result = ERROR_SUCCESS;

	DEBUGMSG( ZONE_USB_CONTROL, (TEXT(">ECAIOUSB_GenericVendorWrite\n")));

	if ( ! VALID_CONTEXT(pContext) )
	{
		DEBUGMSG( ZONE_ERR, (TEXT("Invalid parameter\n")));
		Result = ERROR_INVALID_PARAMETER;
	}
	else
	{
		ControlHeader.bmRequestType = USB_REQUEST_HOST_TO_DEVICE | USB_REQUEST_VENDOR | USB_REQUEST_FOR_DEVICE;
		ControlHeader.bRequest = Request;
		ControlHeader.wValue = Value;
		ControlHeader.wIndex = Index;
		ControlHeader.wLength = (USHORT)*pDataSize;

		dwErr = IssueVendorTransfer(pContext->UsbFuncs, pContext->hUsbDevice, DefaultTransferComplete, pContext->hEP0Event, (USB_OUT_TRANSFER | USB_SHORT_TRANSFER_OK), &ControlHeader, pData, 0, &dwBytesTransferred, pContext->UsbTimeouts.WriteTimeout, &dwUsbErr);

		if ( (dwErr != ERROR_SUCCESS) || (dwUsbErr != USB_NO_ERROR) )
		{
			DEBUGMSG( ZONE_ERR, (TEXT("IssueVendorTransfer ERROR: %d 0x%x\n"), dwErr, dwUsbErr));
			IoErrorHandler(pContext, NULL, 0, dwUsbErr);
			if ( dwErr == ERROR_SUCCESS ) Result = ERROR_GEN_FAILURE; else Result = dwErr;
		}
	}

	DEBUGMSG(ZONE_USB_CONTROL, (TEXT("<ECAIOUSB_GenericVendorWrite: 0x%x\n"), Result));

	return Result;
}


ULONG
EAU_Seek(
   PVOID Context,
   LONG  Position,
   DWORD Type
   )
{
    UNREFERENCED_PARAMETER(Context);
    UNREFERENCED_PARAMETER(Position);
    UNREFERENCED_PARAMETER(Type);
    DEBUGMSG( ZONE_EAU_INIT, (TEXT("EAU_Seek\n")));
    return (ULONG)-1;
}


BOOL
EAU_PowerUp(
   PVOID Context
   )
{
    UNREFERENCED_PARAMETER(Context);
    return 1;
}


BOOL
EAU_PowerDown(
   PVOID Context
   )
{
    UNREFERENCED_PARAMETER(Context);
    return 1;
}


PECAIOUSB_CONTEXT
GetContextFromReg(
   LPTSTR  ActivePath
   )
{
   PECAIOUSB_CONTEXT pContext = NULL;
   HKEY hKey;
   long lStatus;

   if (ActivePath) {
      //
      // open the registry and read out our context pointer
      // since Dev Mgr doesn't pass it in.
      //
      lStatus = RegOpenKeyEx( HKEY_LOCAL_MACHINE,
                              ActivePath,
                              0,
                              0,
                              &hKey);

       if (lStatus==ERROR_SUCCESS) {
         DWORD dwVal;
         DWORD dwType = REG_DWORD;
         DWORD dwValLen = sizeof(dwVal);
         lStatus = RegQueryValueEx( hKey,
                                    TEXT("ClientInfo"),
                                    NULL,
                                    &dwType,
                                    (LPBYTE)(&dwVal),
                                    &dwValLen);

         if (lStatus == ERROR_SUCCESS) {
            // check the signature
            pContext = (PECAIOUSB_CONTEXT)dwVal;
            if ( USB_EAU_SIG != pContext->Sig ) {
               DEBUGMSG(ZONE_ERR, (TEXT("Invalid signature!!\n")));
               TEST_TRAP();
               pContext = NULL;
            } else {
               DEBUGMSG(ZONE_EAU_INIT, (TEXT("ActivePath: %s)\n"), ActivePath));
            }
         }

         RegCloseKey(hKey);

      } else {
         DEBUGMSG(ZONE_ERR, (TEXT("Open ActivePath failed\n")));
      }
   }

   return pContext;
}

/*
Populates the Registry with our EAU Driver Settings
[HKEY_LOCAL_MACHINE\Printers\<DESCRIPTION>
   "Driver"="pcl.dll"      // names the DLL that contains the printer driver
   "High Quality"="300"    // the resolution of high-quality mode
   "Draft Quality"="150   // the resolution of draft-quality mode (optional)
   "Color"="Monochrome"    // literal string Color" -or- "Monochrome"

--*/
BOOL
RegisterECAIOUSBSettings(
   PECAIOUSB_CONTEXT pContext
   )
{

#define MAX_LEN 900

//   char *p;
//   int i;
   BOOL bRc = TRUE;
//   BYTE buf[MAX_LEN];
//   TCHAR tBuffer[MAX_LEN];

   BYTE cDesc[MAX_LEN] = "EAUDevice\\";

//   DWORD dwIdLen,dwIndex;

   REG_VALUE_DESCR keyValues[] = {
      (TEXT("Driver")),        REG_SZ, 0, (PBYTE)(TEXT("pcl.dll")),     // 0 default: WCE currently does not support others, e.g. PostScript
      (TEXT("High Quality")),  REG_SZ, 0, (PBYTE)(TEXT("300")),         // 1 default
      (TEXT("Draft Quality")), REG_SZ, 0, (PBYTE)(TEXT("150")),         // 2 default
      (TEXT("Color")),         REG_SZ, 0, (PBYTE)(TEXT("Monochrome")),  // 3 default
      NULL, 0, 0, NULL                                                  // 4
   };


   DEBUGMSG(ZONE_EAU_INIT, (TEXT(">RegisterECAIOUSBSettings\n")));

   if (!pContext) {
      DEBUGMSG(ZONE_ERR, (TEXT("Invalid parameter\n")));
      return FALSE;
   }

/*
   //
   // read the ID string from the printer
   //
   dwIdLen = GetDeviceId( pContext, buf, MAX_LEN);

   if (dwIdLen) {
      //
      // parse for CMD
      //
      // ...TBD

      //
      // parse for Color support
      //
      p = buf;
      while( (p = strchr(p, '$'))!=NULL ) {
         if (p[2] == 'C') {
            keyValues[3].Data = (PBYTE)(TEXT("Color"));
            break;
         }
         p++;
      }

      //
      // parse for a Description string
      //
      if ((p = strstr(buf, "DESCRIPTION:"))!=NULL)
           p += sizeof("DESCRIPTION:") - 1;

      else if ((p = strstr(buf, "DES:"))!=NULL)
           p += sizeof("DES:") - 1;

      else if ((p = strstr(buf, "MODEL:"))!=NULL)
           p += sizeof("MODEL:") - 1;

      if (p) {
         if (strchr(p, ';')) {
            i = strchr(p, ';') - p;
            p[i] = 0;
         } else {
            i = strlen(p);
         }

         // concat string to \Printers\<DESCRIPTION>
         strncat( cDesc, p, MAX_LEN-1-strlen(cDesc));
        for (dwIndex =0 ; dwIndex< MAX_LEN-1 &&  cDesc[dwIndex]!=0 ;dwIndex++)
            tBuffer[dwIndex]=cDesc[dwIndex];
        tBuffer[dwIndex]=0; // Terminated.

         DEBUGMSG(ZONE_LPT_INIT, (TEXT("DESCRIPTION: %s\n"), tBuffer ));

         //
         // finally, add it to HKEY_LOCAL_MACHINE\Printers\<DESCRIPTION> if it does not already exist.
         //
         bRc = GetSetKeyValues(tBuffer,
                                &keyValues[0],
                                SET,
                                FALSE );

         if ( !bRc ) {
            DEBUGMSG( ZONE_ERR, (TEXT("GetSetKeyValues failed!\n")));
         }

      } else {
         DEBUGMSG(ZONE_ERR, (TEXT("Failed to find DES!\n")));
         bRc = FALSE;
      }

   } else {
      bRc = FALSE;
   }
*/ 
   DEBUGMSG(ZONE_EAU_INIT, (TEXT("<RegisterECAIOUSBSettings:%d\n"), bRc));

   return bRc;
}
//#pragma warning( pop )


//
// Currently leave the info in the registry.
//
BOOL
DeregisterECAIOUSBSettings(
   PECAIOUSB_CONTEXT pContext
   )
{
   BOOL bRc = TRUE;

   if (!pContext) {
      DEBUGMSG(ZONE_ERR, (TEXT("Invalid parameter\n")));
      return FALSE;
   }

   DEBUGMSG(ZONE_EAU_INIT, (TEXT(">DeregisterECAIOUSBSettings\n")));

   // delete any registry data on discconect

   // RegDeleteKey ...

   DEBUGMSG(ZONE_EAU_INIT, (TEXT("<DeregisterECAIOUSBSettings:%d\n"), bRc));

   return bRc;
}

// EOF
