//
// Copyright (c) Microsoft Corporation.  All rights reserved.
//
//
// Use of this source code is subject to the terms of the Microsoft end-user
// license agreement (EULA) under which you licensed this SOFTWARE PRODUCT.
// If you did not accept the terms of the EULA, you are not authorized to use
// this source code. For a copy of the EULA, please see the LICENSE.RTF on your
// install media.
//
/*++

THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF
ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
PARTICULAR PURPOSE.

Module Name: 
    ECAIOUSB.c

Abstract:
    USB Client Driver 
Functions:

Notes: 

--*/

#include "ECAIOUSB.h"
#include "..\inc\EAU.h"

#ifdef DEBUG
DBGPARAM dpCurSettings = {
    TEXT("ECAIOUSB"), {
    TEXT("Errors"),    TEXT("Warnings"),  TEXT("Init"),        TEXT("Trace"),
    TEXT("EAU_INIT"),  TEXT("EAU_READ"),  TEXT("EAU_WRITE"),   TEXT("EAU_IOCTL"),
    TEXT("USB_PARSE"), TEXT("USB_INIT"),  TEXT("USB_CONTROL"), TEXT("USB_BULK"),
    TEXT("Undefined"), TEXT("Undefined"), TEXT("Undefined"),   TEXT("USBCLIENT")  
    },
    0x0307 // ZONE_ERR | ZONE_WAR | ZONE_INIT
};
#endif  // DEBUG


BOOL WINAPI 
DeviceNotify(
   LPVOID lpvNotifyParameter,
   DWORD dwCode,
   LPDWORD * dwInfo1,
   LPDWORD * dwInfo2,
   LPDWORD * dwInfo3,
   LPDWORD * dwInfo4
   );

VOID
RemoveDeviceContext(
   PECAIOUSB_CONTEXT pContext
   );



BOOL 
USBInstallDriver(
   LPCWSTR szDriverLibFile 
   )
{
	HKEY  hKey = NULL;
	BOOL  bRc;

	const WCHAR wsUsbDeviceID[] = CLASS_NAME_SZ;
	WCHAR wsSubClassRegKey[sizeof(CLIENT_REGKEY_SZ)+16] = CLIENT_REGKEY_SZ;


	USB_DRIVER_SETTINGS usbDriverSettings = { DRIVER_SETTINGS };
	USB_DRIVER_SETTINGS usbDriverSettingsPre = { DRIVER_SETTINGS_PRE };


    DWORD dwDefaultTimeout  = DEFAULT_TIMEOUT;
    DWORD dwReadTimeout    = READ_TIMEOUT;
    DWORD dwWriteTimeout   = WRITE_TIMEOUT;

    REG_VALUE_DESCR ECAIOUSBKeyValues[] = {
        (TEXT("Dll")),              REG_SZ,    0, (PBYTE)(DRIVER_NAME),
        (TEXT("Prefix")),           REG_SZ,    0, (PBYTE)(DEVICE_PREFIX),
        DEFAULT_TIMEOUT_SZ,         REG_DWORD, 0, (PUCHAR)(&dwDefaultTimeout),
        READ_TIMEOUT_SZ,			REG_DWORD, 0, (PUCHAR)(&dwReadTimeout),
        WRITE_TIMEOUT_SZ,			REG_DWORD, 0, (PUCHAR)(&dwWriteTimeout),
        NULL, 0, 0, NULL
    };

   DEBUGMSG( ZONE_USB_INIT, (TEXT(">USBInstallDriver(%s)\n"), szDriverLibFile ));
    
    //
   // register with USBD
   //   
   bRc = RegisterClientDriverID( wsUsbDeviceID );
   if ( !bRc ) {
      DEBUGMSG( ZONE_ERR, (TEXT("RegisterClientDriverID error:%d\n"), GetLastError()));
      return FALSE;
   }
        
   bRc = RegisterClientSettings( szDriverLibFile,
                                 wsUsbDeviceID, 
                                 NULL, 
                                 &usbDriverSettings );
   if ( !bRc ) {
      DEBUGMSG( ZONE_ERR, (TEXT("RegisterClientSettings error:%d\n"), GetLastError()));
      return FALSE;
   }

   bRc = RegisterClientSettings( szDriverLibFile,
                                 wsUsbDeviceID, 
                                 NULL, 
                                 &usbDriverSettingsPre );
   if ( !bRc ) {
      DEBUGMSG( ZONE_ERR, (TEXT("RegisterClientSettings error:%d\n"), GetLastError()));
      return FALSE;
   }

    //
    // Add our default values to the reg
    //
    if ( !GetSetKeyValues( wsSubClassRegKey,
                           &ECAIOUSBKeyValues[0],
                           SET,
                           TRUE ) ) {
        DEBUGMSG( ZONE_ERR, (TEXT("GetSetKeyValues failed!\n")));
        TEST_TRAP();
    }
 
    DEBUGMSG( ZONE_USB_INIT, (TEXT("<USBInstallDriver:%d\n"), bRc ));

   return bRc;
}


BOOL 
USBUnInstallDriver(
   VOID
   )
{
	BOOL bRc;
	const WCHAR wsUsbDeviceID[] = CLASS_NAME_SZ;
	USB_DRIVER_SETTINGS usbDriverSettings = { DRIVER_SETTINGS };
	USB_DRIVER_SETTINGS usbDriverSettingsPre = { DRIVER_SETTINGS_PRE };

   DEBUGMSG( ZONE_USB_INIT, (TEXT(">USBUnInstallDriver\n")));

   bRc = UnRegisterClientSettings( wsUsbDeviceID,
                                   NULL,
                                   &usbDriverSettings );

   bRc = bRc & UnRegisterClientSettings( wsUsbDeviceID,
                                   NULL,
                                   &usbDriverSettingsPre );

   bRc = bRc & UnRegisterClientDriverID( wsUsbDeviceID );

   DEBUGMSG( ZONE_USB_INIT, (TEXT("<USBUnInstallDriver:%d\n"), bRc));

   return bRc;
}


BOOL
USBDeviceAttach(
   USB_HANDLE       hDevice,
   LPCUSB_FUNCS     UsbFuncs,      
   LPCUSB_INTERFACE UsbInterface,
   LPCWSTR          UniqueDriverId,   
   LPBOOL           AcceptControl,      
   LPCUSB_DRIVER_SETTINGS UsbDriverSettings,
   DWORD Unused
    )
{
   BOOL bRc = TRUE;
   PECAIOUSB_CONTEXT  pContext = NULL;
   LPCUSB_INTERFACE pUsbInterface = NULL;
   ULONG ulConfigIndex = 0;

   WCHAR wsSubClassRegKey[sizeof(CLIENT_REGKEY_SZ)+16] = CLIENT_REGKEY_SZ;

   REG_VALUE_DESCR rdTimeouts[] = {
        DEFAULT_TIMEOUT_SZ, REG_DWORD, sizeof(DWORD), (PUCHAR)(NULL),
        READ_TIMEOUT_SZ,    REG_DWORD, sizeof(DWORD), (PUCHAR)(NULL),
        WRITE_TIMEOUT_SZ,   REG_DWORD, sizeof(DWORD), (PUCHAR)(NULL),
        NULL, 0, 0, NULL
   };

   UNREFERENCED_PARAMETER(UniqueDriverId);
   UNREFERENCED_PARAMETER(UsbDriverSettings);
   UNREFERENCED_PARAMETER(Unused);

	DEBUGMSG( ZONE_USB_INIT, (TEXT(">USBDeviceAttach(0x%x, %s)\n"), hDevice, UniqueDriverId));

	//
	// Determine if we control this USB peripheral...
	//
	*AcceptControl = FALSE;

	do {
		//TODO check for valid USB devie VID/PID or Interface 
		//
		// Parse USB Descriptors
		//
		if (!ParseUsbDescriptors(hDevice,
								UsbFuncs,
								UsbInterface,
								(LPUSHORT)&ulConfigIndex,
								pUsbInterface)) {
			DEBUGMSG(ZONE_ERR, (TEXT("ParseUsbDescriptors failed!\n")));
			bRc = FALSE;
			break;
		}
	   
		 //
		 // we found a device & interface we control, so create our device context
		 //
		 pContext = (PECAIOUSB_CONTEXT)LocalAlloc( LPTR, sizeof(ECAIOUSB_CONTEXT) );
		 if ( !pContext ) {
			DEBUGMSG( ZONE_ERR, (TEXT("LocalAlloc error:%d\n"), GetLastError() ));
			bRc = FALSE;
			break;
		 }

		 pContext->Sig = USB_EAU_SIG;

		 InitializeCriticalSection( &pContext->Lock );

		 pContext->hUsbDevice = hDevice;

		 pContext->UsbFuncs = UsbFuncs;
	  
		 pContext->Flags.Open = FALSE;
		 pContext->Flags.UnloadPending = FALSE;

		 // create endpoint 0 event
		 pContext->hEP0Event = CreateEvent( NULL, TRUE, FALSE, NULL);
		 if ( !pContext->hEP0Event ) {
			 DEBUGMSG( ZONE_ERR, (TEXT("CreateEvent error:%d\n"), GetLastError() ));
			 bRc = FALSE;
			 break;
		 }

		 pContext->hCloseEvent = CreateEvent( NULL, FALSE, FALSE, NULL);
		 if ( !pContext->hCloseEvent ) {
			DEBUGMSG( ZONE_ERR, (TEXT("CreateEvent error:%d\n"), GetLastError() ));
			bRc = FALSE;
			break;
		}

		if (pUsbInterface) {
			//
			// set the USB interface/pipes
			//
			bRc = SetUsbInterface( pContext, 
								 pUsbInterface );
			if ( !bRc ) {
				DEBUGMSG( ZONE_ERR, (TEXT("SetUsbInterface failed!\n")));
				break;
			}
		}

		//
		// Read the timeout values from the registry
		//
		rdTimeouts[0].Data = (PUCHAR)(&pContext->UsbTimeouts.DefaultTimeout);
		rdTimeouts[1].Data = (PUCHAR)(&pContext->UsbTimeouts.ReadTimeout);
		rdTimeouts[2].Data = (PUCHAR)(&pContext->UsbTimeouts.WriteTimeout);
	        
		if (!GetSetKeyValues(wsSubClassRegKey,
							 &rdTimeouts[0],
							 GET, 
							 FALSE) ) {
			//
			// use defaults
			//
			pContext->UsbTimeouts.DefaultTimeout  = DEFAULT_TIMEOUT;
			pContext->UsbTimeouts.ReadTimeout    = READ_TIMEOUT;
			pContext->UsbTimeouts.WriteTimeout   = WRITE_TIMEOUT;
		}

		//
		// kick start the stream driver interface
		//
		pContext->hStreamDevice = ActivateDevice( wsSubClassRegKey, (DWORD)pContext );

		if ( pContext->hStreamDevice ) {
			//
			// register for USB callbacks
			//
			bRc = UsbFuncs->lpRegisterNotificationRoutine( hDevice, DeviceNotify, pContext );
			if ( !bRc ) {
				DEBUGMSG( ZONE_ERR, (TEXT("RegisterNotificationRoutine error:%d\n"), GetLastError() ));
				break;
			}
		} else {
			//
			// the streams interface failed to init, no use starting.
			//
			DEBUGMSG( ZONE_ERR, (TEXT("ActivateDevice error:%d\n"), GetLastError() ));
			bRc = FALSE;
			break;
		}
		
	} while(0);

	if (!bRc) {
		//
		// If not our device, or error, then clean up
		//
		RemoveDeviceContext( pContext );
	} else {
      *AcceptControl = TRUE;   
	}
	// optional used for DeviceNotifcation Add/remove
	AdvertiseInterface(&EAU_guid, L"EAU1:", TRUE);

	DEBUGMSG( ZONE_USB_INIT, (TEXT("<USBDeviceAttach:%d\n"), *AcceptControl ));

	return bRc;
}


BOOL
ParseUsbDescriptors(
   USB_HANDLE       hUsbDevice,
   LPCUSB_FUNCS     UsbFuncs,
   LPCUSB_INTERFACE CurInterface,
   LPUSHORT         ConfigIndex,
   LPCUSB_INTERFACE pUsbInterface
   )
{
	BOOL				bRet = FALSE;
	LPCUSB_DEVICE		pDevice;
	LPCUSB_INTERFACE	pDesiredInterface = NULL;
   
	//DWORD dwNumInterfaces;
	//DWORD bProtocol, dwIndex;

	DEBUGMSG( ZONE_USB_INIT, (TEXT(">ParseUsbDescriptors\n")));

	//if ( !hUsbDevice || !UsbFuncs || !CurInterface || !ConfigIndex) {
	if ( !hUsbDevice || !UsbFuncs ) {
		DEBUGMSG( ZONE_ERR, (TEXT("Invalid parameter\n")));
		return bRet;
	}

	//
	// get the descriptors
	//
	pDevice = UsbFuncs->lpGetDeviceInfo( hUsbDevice );
	if ( !pDevice ) {
		DEBUGMSG( ZONE_ERR, (TEXT("GetDeviceInfo error:%d\n"), GetLastError() ));
		return bRet;
	}

	DUMP_USB_DEVICE_DESCRIPTOR( pDevice->Descriptor );
	DUMP_USB_CONFIGURATION_DESCRIPTOR( pDevice->lpActiveConfig->Descriptor );  

	//Perform any device verification or interface selection 

	// get config index
	for (*ConfigIndex = 0; *ConfigIndex < (USHORT)pDevice->Descriptor.bNumConfigurations; *ConfigIndex++) {
		if (pDevice->lpActiveConfig == (pDevice->lpConfigs + *ConfigIndex)) {
			DEBUGMSG(ZONE_USB_INIT, (TEXT("ConfigIndex:%d\n"), *ConfigIndex));
			break;
		}
	}


	if (
		(pDevice->Descriptor.idVendor == USB_EAU_VENDOR_ID)
		&& ((pDevice->Descriptor.idProduct == USB_EAU_PRODUCT_ID) || (pDevice->Descriptor.idProduct == USB_EAU_PRODUCT_ID_PRE)) //May want to remove PID checking entirely. They can all be used through FWE 2.0 abstraction and such.
	) {

		RETAILMSG(1, (TEXT("ParseUsbDescriptors: - pDevice 0x%x VID 0x%x(%d) PID 0x%x(%d)\n"), 
			pDevice, pDevice->Descriptor.idVendor, pDevice->Descriptor.idVendor, 
			pDevice->Descriptor.idProduct, pDevice->Descriptor.idProduct));
		bRet = TRUE;
	}

/*
   pUsbInterface   = pDevice->lpActiveConfig->lpInterfaces;
   dwNumInterfaces = pDevice->lpActiveConfig->dwNumInterfaces;

   // walk the interfaces searching for best fit
   for ( bProtocol = 0, dwIndex = 0; dwIndex < dwNumInterfaces; pUsbInterface++, dwIndex++) {

      DUMP_USB_INTERFACE_DESCRIPTOR( pUsbInterface->Descriptor, dwIndex );

  TODO determine if this is a device we can control!!!!!!
		if ( pUsbInterface->Descriptor.bInterfaceNumber == CurInterface->Descriptor.bInterfaceNumber ) {
            if (pUsbInterface->Descriptor.bInterfaceClass     == USB_PRN_INTERFACE_CLASS &&
                pUsbInterface->Descriptor.bInterfaceSubClass  == USB_PRN_INTERFACE_SUBCLASS &&
                (pUsbInterface->Descriptor.bInterfaceProtocol == USB_PRN_INTERFACE_PROTOCOL_BI || 
                pUsbInterface->Descriptor.bInterfaceProtocol  == USB_PRN_INTERFACE_PROTOCOL_UNI) ) 
         {
            if (pUsbInterface->Descriptor.bInterfaceProtocol > bProtocol) {
                   pDesiredInterface = pUsbInterface;
                    bProtocol = pUsbInterface->Descriptor.bInterfaceProtocol;
               DEBUGMSG( ZONE_USB_INIT, (TEXT("*** Found interface @ index: %d ***\n"), dwIndex));
               }
            }
        }
*/


  
   DEBUGMSG( ZONE_USB_INIT, (TEXT("<ParseUsbDescriptors:0x%x\n"), bRet ));

   return bRet;
}


BOOL
SetUsbInterface(
   PECAIOUSB_CONTEXT  pContext,
   LPCUSB_INTERFACE UsbInterface
   )
{
   //USB_TRANSFER hTransfer;
   BOOL bRc = FALSE;
   DWORD dwIndex;

   DEBUGMSG( ZONE_USB_INIT, (TEXT(">SetUsbInterface\n")));

/*TODO
   if ( !VALID_CONTEXT( pContext ) || !UsbInterface ) {
      DEBUGMSG( ZONE_ERR, (TEXT("Invalid parameter\n")));
      return FALSE;
   }

   hTransfer = pContext->UsbFuncs->lpSetInterface( pContext->hUsbDevice,
                                                  NULL,
                                                  NULL,
                                                  0, // synchronous
                                                  UsbInterface->Descriptor.bInterfaceNumber,
                                                  UsbInterface->Descriptor.bAlternateSetting );

   if ( !hTransfer ) {
      DEBUGMSG( ZONE_ERR, (TEXT("SetUsbInterface error:%d\n"), GetLastError() ));
      return FALSE;
   }
*/
   //
   // now parse the endpoints
   //
    for ( dwIndex = 0; dwIndex < UsbInterface->Descriptor.bNumEndpoints; dwIndex++) {
    
      LPCUSB_ENDPOINT pEndpoint;
      
      pEndpoint = UsbInterface->lpEndpoints + dwIndex;

      DUMP_USB_ENDPOINT_DESCRIPTOR(pEndpoint->Descriptor);

/* TODO check EAU endpoints
    // 
    // Printer Class supports 1 mandatory Bulk OUT and 1 optional Bulk IN
    // 
    if ( USB_ENDPOINT_DIRECTION_OUT( pEndpoint->Descriptor.bEndpointAddress ) ) {
            if ( NULL == pContext->BulkOut.hPipe  &&
              (pEndpoint->Descriptor.bmAttributes & USB_ENDPOINT_TYPE_MASK) == USB_ENDPOINT_TYPE_BULK) {
               //
               // create the Bulk OUT pipe
               //
               pContext->BulkOut.hPipe = pContext->UsbFuncs->lpOpenPipe( pContext->hUsbDevice,
                                                                  &pEndpoint->Descriptor );
               if ( !pContext->BulkOut.hPipe ) {
                  DEBUGMSG( ZONE_ERR, (TEXT("OpenPipe error:%d\n"), GetLastError() ));
                  bRc = FALSE;
                  TEST_TRAP();
                  break;
               }

               //
               // setup any endpoint specific timers, buffers, context, etc.
               //
               pContext->BulkOut.hEvent = CreateEvent( NULL, MANUAL_RESET_EVENT, FALSE, NULL);
               if ( !pContext->BulkOut.hEvent ) {
                  DEBUGMSG( ZONE_ERR, (TEXT("CreateEvent error:%d\n"), GetLastError() ));
                  bRc = FALSE;
                  TEST_TRAP();
                  break;
               }

               pContext->BulkOut.bIndex = pEndpoint->Descriptor.bEndpointAddress;
         }

        } else if (USB_ENDPOINT_DIRECTION_IN( pEndpoint->Descriptor.bEndpointAddress ) ) {
         if ( NULL == pContext->BulkIn.hPipe && 
              USB_PRN_INTERFACE_PROTOCOL_BI == UsbInterface->Descriptor.bInterfaceProtocol &&
                  (pEndpoint->Descriptor.bmAttributes & USB_ENDPOINT_TYPE_MASK) == USB_ENDPOINT_TYPE_BULK) {
               //
               // create the Bulk IN pipe
               //
               pContext->BulkIn.hPipe = pContext->UsbFuncs->lpOpenPipe( pContext->hUsbDevice,
                                                                 &pEndpoint->Descriptor );
               if ( !pContext->BulkIn.hPipe ) {
                  DEBUGMSG( ZONE_ERR, (TEXT("OpenPipe error: %d\n"), GetLastError() ));
                  bRc = FALSE;
                  TEST_TRAP();
                  break;
               }

               //
               // setup any endpoint specific timers, buffers, context, etc.
               //
               pContext->BulkIn.hEvent = CreateEvent( NULL, MANUAL_RESET_EVENT, FALSE, NULL);
               if ( !pContext->BulkIn.hEvent ) {
                  DEBUGMSG( ZONE_ERR, (TEXT("CreateEvent error:%d\n"), GetLastError() ));
                  bRc = FALSE;
                  TEST_TRAP();
                  break;
               }

               pContext->BulkIn.bIndex = pEndpoint->Descriptor.bEndpointAddress;
            }

      } else {
         DEBUGMSG( ZONE_WARN, (TEXT("Unsupported Endpoint:0x%x\n"), pEndpoint->Descriptor.bEndpointAddress));
      }
*/

    }

/*
   //
   // did we find our endpoints?
   //
    if ( USB_PRN_INTERFACE_PROTOCOL_UNI == UsbInterface->Descriptor.bInterfaceProtocol ) {
      
      bRc = pContext->BulkOut.hPipe ? TRUE : FALSE;
   
   } else {
      
      bRc = (pContext->BulkOut.hPipe && pContext->BulkIn.hPipe) ? TRUE : FALSE;
   }

   CloseTransferHandle(pContext->UsbFuncs, hTransfer);

   //
   // if we failed to find all of our endpoints then cleanup will occur later
   //
*/

    DEBUGMSG( ZONE_USB_INIT, (TEXT("<SetUsbInterface:%d\n"), bRc));

    return (bRc);
}


BOOL WINAPI 
DeviceNotify(
   LPVOID lpvNotifyParameter,
   DWORD dwCode,
   LPDWORD * dwInfo1,
   LPDWORD * dwInfo2,
   LPDWORD * dwInfo3,
   LPDWORD * dwInfo4
   )
{
   PECAIOUSB_CONTEXT pContext = (PECAIOUSB_CONTEXT )lpvNotifyParameter;
   DWORD dwWaitReturn;
   BOOL bRc;

    UNREFERENCED_PARAMETER(dwInfo1);
    UNREFERENCED_PARAMETER(dwInfo2);
    UNREFERENCED_PARAMETER(dwInfo3);
    UNREFERENCED_PARAMETER(dwInfo4);

   DEBUGMSG( ZONE_USB_INIT, (TEXT(">DeviceNotify\n")));

   if ( !VALID_CONTEXT( pContext ) ) {
      DEBUGMSG( ZONE_ERR, (TEXT("Invalid Context!\n")));
      return FALSE;
   }
   
   switch(dwCode) {

      case USB_CLOSE_DEVICE:
        DEBUGMSG( ZONE_USB_INIT, (TEXT("USB_CLOSE_DEVICE\n")));
		if (pContext->Flags.Open) {
           //
           // When an app has an open handle to this device and then the user unplugs the USB cable
           // the driver will unload here. However, the app can a) hang on a removed handle, or
           // b) pass a call to a removed function, e.g., CloseHandle and cause an AV.
           // Set state to reject all I/O and wait for cleanup signal.
           //
           EnterCriticalSection( &pContext->Lock );
           pContext->Flags.Open = FALSE;
           pContext->Flags.UnloadPending = TRUE;
           LeaveCriticalSection( &pContext->Lock );

		   // optional deregistry DeviceNotifications
		   AdvertiseInterface(&EAU_guid, L"EAU1:", FALSE);


           DEBUGMSG( ZONE_USB_INIT, (TEXT("Waiting for CloseEvent...\n")));
           //dwWaitReturn = WaitForSingleObject( pContext->hCloseEvent, INFINITE );

		   // only wait 10 seconds.....
           dwWaitReturn = WaitForSingleObject( pContext->hCloseEvent, 10000 );

           switch (dwWaitReturn) {

               case WAIT_OBJECT_0:
                  DEBUGMSG( ZONE_USB_INIT, (TEXT("...CloseEvent signalled\n")));
                  break;

               case WAIT_FAILED:
                  DEBUGMSG( ZONE_ERR, (TEXT("CloseEvent error:%d\n"), GetLastError()));
                  TEST_TRAP();
                  break;
               
               default:
                  DEBUGMSG( ZONE_ERR, (TEXT("Unhandled WaitReason:%d\n"), dwWaitReturn ));
                  TEST_TRAP();
                  break;
            }
        
           //
           // Warning: DeactivateDevice forces Device Manager to call our EAU_Deinit, which then
           // causes COREDLL!xxX_CloseHandle to AV when PRNPORT!PrinterSend calls CloseHandle(hPrinter).
           // So, give it a chance to clean up.
           //
           Sleep(1000);
		} else {
			// optional deregistry DeviceNotifications
			AdvertiseInterface(&EAU_guid, L"EAU1:", FALSE);
		}

        DEBUGMSG( ZONE_USB_INIT, (TEXT("DeactivateDevice\n")));

        bRc = DeactivateDevice( pContext->hStreamDevice );

        if ( !bRc ) {
            DEBUGMSG( ZONE_ERR, (TEXT("DeactivateDevice error: %d\n"), GetLastError() ));
            TEST_TRAP();
        }

        RemoveDeviceContext( pContext );

        return TRUE;

      default:
         DEBUGMSG( ZONE_ERR, (TEXT("Unhandled code:%d\n"), dwCode));
         TEST_TRAP();
         break;
    }
   
   DEBUGMSG( ZONE_USB_INIT, (TEXT("<DeviceNotify\n")));
   
   return FALSE;
}


/*++
Class specific command:
   GET_DEVICE_ID

Return: number of bytesin the Device ID string.
--*/
DWORD
GetDeviceId(
   PECAIOUSB_CONTEXT pContext,
   PBYTE           pBuffer,
   DWORD           dwLen
   )
{
    USB_DEVICE_REQUEST ControlHeader;
    DWORD dwBytesTransferred = 0;
    DWORD dwErr = ERROR_SUCCESS;
    DWORD dwUsbErr = USB_NO_ERROR;

    DEBUGMSG( ZONE_USB_CONTROL, (TEXT(">GetDeviceId\n")));

    if ( !VALID_CONTEXT( pContext ) || !pBuffer || dwLen <= 2) {
        DEBUGMSG( ZONE_ERR, (TEXT("Invalid parameter\n")));
    } else {
   
        ControlHeader.bmRequestType = USB_REQUEST_DEVICE_TO_HOST | USB_REQUEST_CLASS | USB_REQUEST_FOR_INTERFACE;
        ControlHeader.bRequest = 0;
        ControlHeader.wValue = pContext->ConfigIndex;
        ControlHeader.wIndex = pContext->bInterfaceNumber*0x100 + pContext->bAlternateSetting;
        ControlHeader.wLength = (USHORT)dwLen;

        dwErr = IssueVendorTransfer( pContext->UsbFuncs,
                                     pContext->hUsbDevice,
                                     DefaultTransferComplete,
                                     pContext->hEP0Event,
                                     (USB_IN_TRANSFER | USB_SHORT_TRANSFER_OK),
                                    &ControlHeader,
                                     pBuffer, 0,
                                    &dwBytesTransferred,
                                     pContext->UsbTimeouts.DefaultTimeout,
                                    &dwUsbErr );
      
      if ( ERROR_SUCCESS == dwErr && USB_NO_ERROR == dwUsbErr && dwBytesTransferred ) {
         //
         // remove the 2 byte IEEE-1284 Device ID string length for Port Monitor
         //
          if ( dwBytesTransferred > 2) {
              dwBytesTransferred -= 2;
              memmove( pBuffer, pBuffer + 2, dwBytesTransferred);
              pBuffer[dwBytesTransferred] = 0;
          } else {
              dwBytesTransferred = 0;
          };

      } else {
         DEBUGMSG( ZONE_ERR, (TEXT("IssueVendorTransfer ERROR:%d 0x%x\n"), dwErr, dwUsbErr));
         IoErrorHandler( pContext, NULL, 0, dwUsbErr);
      }
   }

    DEBUGMSG( ZONE_USB_CONTROL, (TEXT("<GetDeviceId:%d\n"), dwBytesTransferred));

   return dwBytesTransferred;
}


/*++
Class specific command:
   GET_PORT_STATUS

Return: printer status bits.
--*/
USHORT
GetPortStatus( 
   PECAIOUSB_CONTEXT pContext
   )
{
    USB_DEVICE_REQUEST ControlHeader;
    DWORD  dwBytesTransferred = 0;
    DWORD  dwUsbErr = USB_NO_ERROR;
    DWORD  dwErr = ERROR_SUCCESS;
    USHORT usStatus = 0;

    DEBUGMSG( ZONE_USB_CONTROL, (TEXT(">GetPortStatus\n")));

    if ( !VALID_CONTEXT( pContext)  ) {
        DEBUGMSG( ZONE_ERR, (TEXT("Invalid parameter\n")));
    } else {

        ControlHeader.bmRequestType = USB_REQUEST_DEVICE_TO_HOST | USB_REQUEST_CLASS | USB_REQUEST_FOR_INTERFACE;
        ControlHeader.bRequest = 1;
        ControlHeader.wValue   = 0;
        ControlHeader.wIndex   = pContext->bInterfaceNumber;
        ControlHeader.wLength  = 1;

        dwErr = IssueVendorTransfer( pContext->UsbFuncs,
                                     pContext->hUsbDevice,
                                     DefaultTransferComplete,
                                     pContext->hEP0Event,
                                     (USB_IN_TRANSFER | USB_SHORT_TRANSFER_OK),
                                     &ControlHeader,
                                     &usStatus, 0,
                                     &dwBytesTransferred,
                                     pContext->UsbTimeouts.DefaultTimeout,
                                     &dwUsbErr );

        if ( ERROR_SUCCESS != dwErr || USB_NO_ERROR != dwUsbErr) {
            DEBUGMSG( ZONE_ERR, (TEXT("IssueVendorTransfer ERROR:%d 0x%x\n"), dwErr, dwUsbErr));
            IoErrorHandler( pContext, NULL, 0, dwUsbErr);
        }
    }

    DEBUGMSG( ZONE_USB_CONTROL, (TEXT("<GetPortStatus:0x%x\n"), usStatus));

    return usStatus;
}


/*++
Class specific command:
   SOFT_RESET

Return: TRUE if successful, else FALSE.

Note: SoftReset currently only works on the Epson Stylus Color 740,
but times out on the HP Deskjet 810C. H/W problem on the HP?

--*/
BOOL
SoftReset( 
   PECAIOUSB_CONTEXT pContext
   )
{
    USB_DEVICE_REQUEST ControlHeader;
    DWORD  dwBytesTransferred = 0;
    DWORD  dwUsbErr = USB_NO_ERROR;
    DWORD  dwErr = ERROR_SUCCESS;
    USHORT usStatus = 0;
    BOOL bRc = TRUE;

    DEBUGMSG( ZONE_USB_CONTROL, (TEXT(">SoftReset\n")));

    if ( !VALID_CONTEXT( pContext) ) {
        DEBUGMSG( ZONE_ERR, (TEXT("Invalid parameter\n")));
        bRc = FALSE;
   } else {

        ControlHeader.bmRequestType = USB_REQUEST_HOST_TO_DEVICE | USB_REQUEST_CLASS | USB_REQUEST_FOR_OTHER;
        ControlHeader.bRequest = 2;
        ControlHeader.wValue   = 0;
        ControlHeader.wIndex   = pContext->bInterfaceNumber;
        ControlHeader.wLength  = 0;

        dwErr = IssueVendorTransfer( pContext->UsbFuncs,
                                     pContext->hUsbDevice,
                                     DefaultTransferComplete,
                                     pContext->hEP0Event,
                                     (USB_OUT_TRANSFER | USB_SHORT_TRANSFER_OK),
                                     &ControlHeader,
                                     NULL, 0,
                                     &dwBytesTransferred,
                                     pContext->UsbTimeouts.DefaultTimeout,
                                     &dwUsbErr );

        if ( ERROR_SUCCESS != dwErr || USB_NO_ERROR != dwUsbErr) {
            DEBUGMSG( ZONE_ERR, (TEXT("IssueVendorTransfer ERROR:%d 0x%x\n"), dwErr, dwUsbErr));
            IoErrorHandler( pContext, NULL, 0, dwUsbErr);
        }
    }

    DEBUGMSG( ZONE_USB_CONTROL, (TEXT("<SoftReset:%d\n"), bRc));

    return bRc;
}


//
// returns Win32 error
//
DWORD 
IoErrorHandler( 
   PECAIOUSB_CONTEXT pContext,
   USB_PIPE        hPipe, // if NULL then uses default endpoint,
   UCHAR           bIndex,
   USB_ERROR       dwUsbErr
   )
{
   DWORD dwErr = ERROR_SUCCESS;

   DEBUGMSG(ZONE_ERR|ZONE_TRACE,(TEXT(">IoErrorHandler(%p, 0x%x)\n"), hPipe, dwUsbErr));

   if ( VALID_CONTEXT( pContext ) ) {

      switch (dwUsbErr) {

         case USB_STALL_ERROR:
            DEBUGMSG(ZONE_ERR,(TEXT("USB_STALL_ERROR\n")));
            if (hPipe) {
               //
               // reset the pipe in the USB stack
               //
               if (ResetPipe( pContext->UsbFuncs, hPipe, 0) )
                  //
                  // clear the stall on the device
                  //
                  dwErr = ClearOrSetFeature( pContext->UsbFuncs,
                                             pContext->hUsbDevice,
                                             DefaultTransferComplete,
                                             pContext->hEP0Event,
                                             USB_SEND_TO_ENDPOINT,
                                             USB_FEATURE_ENDPOINT_STALL,
                                             bIndex,
                                             1000,
                                             FALSE );
            } else {
               //
               // reset the default endpoint
               //
               dwErr = ResetDefaultEndpoint(pContext->UsbFuncs, pContext->hUsbDevice);
            
            }
            break;

         case USB_DEVICE_NOT_RESPONDING_ERROR:
            DEBUGMSG(ZONE_ERR,(TEXT("USB_DEVICE_NOT_RESPONDING_ERROR\n")));
            //
            // try a SoftReset
            //
            if ( !SoftReset(pContext) ) {
               //
               // If the device is not responding and we can not soft reset then assume it's dead.
               // WCE can't reset the port yet.
               //
               DEBUGMSG( ZONE_ERR, (TEXT("Dead device\n")));
            }
            break;

         default:
            DEBUGMSG(ZONE_ERR,(TEXT("Unhandled error: 0x%x\n"), dwUsbErr));
            break;
      }

   } else {
      dwErr = ERROR_INVALID_PARAMETER;
      DEBUGMSG(ZONE_ERR,(TEXT("Invalid Parameter\n")));
   }

   DEBUGMSG(ZONE_ERR|ZONE_TRACE,(TEXT("<IoErrorHandler:%d\n"), dwErr));

   return dwErr;
}


BOOL 
DllEntry(
   HANDLE hDllHandle,
   DWORD  dwReason,
   LPVOID lpreserved
   ) 
{
    UNREFERENCED_PARAMETER(hDllHandle);
    UNREFERENCED_PARAMETER(lpreserved);
    switch (dwReason) {

      case DLL_PROCESS_ATTACH:
           DEBUGREGISTER((HINSTANCE)hDllHandle);
	   DisableThreadLibraryCalls((HMODULE) hDllHandle);
           break;

      case DLL_PROCESS_DETACH:
           break;

      default:
        break;
    }
    return TRUE;
}


VOID
RemoveDeviceContext(
   PECAIOUSB_CONTEXT pContext
   )
{
   DEBUGMSG(ZONE_INIT,(TEXT(">RemoveDeviceContext(%p)\n"), pContext));

   if ( VALID_CONTEXT( pContext ) ) {

      if ( pContext->Flags.Open ) {
         DEBUGMSG(ZONE_ERR,(TEXT("RemoveDeviceContext on open device!\n")));
         TEST_TRAP();
         return;
      }

      if (pContext->BulkIn.hPipe && pContext->UsbFuncs) {
         pContext->UsbFuncs->lpClosePipe(pContext->BulkIn.hPipe);
      }
      
      if (pContext->BulkIn.hEvent) {
         CloseHandle(pContext->BulkIn.hEvent);
      }
      
      if (pContext->BulkOut.hPipe && pContext->UsbFuncs) {
         pContext->UsbFuncs->lpClosePipe(pContext->BulkOut.hPipe);
      }

      if (pContext->hEP0Event) {
         CloseHandle(pContext->hEP0Event);
      }

      if (pContext->BulkOut.hEvent) {
         CloseHandle(pContext->BulkOut.hEvent);
      }

      if (pContext->hCloseEvent) {
         CloseHandle(pContext->hCloseEvent);
      }
      
      if (&pContext->Lock) {
         DeleteCriticalSection( &pContext->Lock );
      }
      
      LocalFree(pContext);
   
   } else {
      DEBUGMSG(ZONE_ERR,(TEXT("Invalid Parameter\n")));
   }

   DEBUGMSG(ZONE_INIT,(TEXT("<RemoveDeviceContext\n")));
   
   return;
}




// EOF
