//
// Copyright (c) Microsoft Corporation.  All rights reserved.
//
//
// Use of this source code is subject to the terms of the Microsoft end-user
// license agreement (EULA) under which you licensed this SOFTWARE PRODUCT.
// If you did not accept the terms of the EULA, you are not authorized to use
// this source code. For a copy of the EULA, please see the LICENSE.RTF on your
// install media.
//
/*++

THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF
ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
PARTICULAR PURPOSE.

Module Name:
    ECAIOUSB.h

Abstract:
    USB Client Driver for Device Class.

Functions:

Notes:

--*/

#if !defined( _ECAIOUSB_ )
#define _ECAIOUSB_

#include <windows.h>
#include <devload.h>
#include <usbdi.h>
//#include <pegdser.h>
//#include <pegdpar.h>

#include <ntcompat.h>

#include "usbclient.h"


#define DRIVER_NAME   TEXT("ECAIOUSB.DLL")
#define DEVICE_PREFIX TEXT("EAU")

#define CLASS_NAME_SZ    TEXT("ECAIOUSB")
#define CLIENT_REGKEY_SZ TEXT("Drivers\\USB\\ClientDrivers\\ECAIOUSB")


//
// USB  Interface Descriptor
//
#define USB_EAU_VENDOR_ID        0x1605
#define USB_EAU_PRODUCT_ID      0x8010
#define USB_EAU_PRODUCT_ID_PRE  0x0010


//
// Default Timeout values
//
#define DEFAULT_TIMEOUT	1000
#define READ_TIMEOUT	5000
#define WRITE_TIMEOUT	1000

//
// registry strings
//
#define DEFAULT_TIMEOUT_SZ	TEXT("DefaultTimeout")
#define READ_TIMEOUT_SZ		TEXT("ReadTimeout")
#define WRITE_TIMEOUT_SZ	TEXT("WriteTimeout")


#define USB_EAU_SIG 0x78425355 // "USBx" tag


typedef struct _FLAGS {
   UCHAR    Open           : 1; // bits 0
   UCHAR    UnloadPending  : 1; // bits 1
   UCHAR    Reserved       : 6; // bits 2-7
} FLAGS, *PFLAGS;

typedef struct _USBTIMEOUTS {

    DWORD DefaultTimeout;

    DWORD ReadTimeout;

    DWORD WriteTimeout;

} USBTIMEOUTS, *PUSBTIMEOUTS;

//
// Our notion of a Pipe
//
typedef struct _PIPE {
    //
    // USB Pipe handle received from the stack
    //
    USB_PIPE hPipe;

    //
    // Endpoint's Address
    //
    UCHAR    bIndex;

    //
    // Endpoint's wMaxPacketSize
    //
    USHORT wMaxPacketSize;

    //
    // Completion Event
    //
    HANDLE   hEvent;

} PIPE, *PPIPE;


typedef struct _ECAIOUSB_CONTEXT {
   //
   // We use a Signature (defined above) since we must read
   // our context pointer out of the registry
   //
   ULONG Sig;

   //
   // sync object for this instance
   //
   CRITICAL_SECTION Lock;

   //
   // path to HKLM\Drivers\Active\xx
   //
   LPTSTR  ActivePath;

   //
   // Handle for Stream interface.
   //
   HANDLE   hStreamDevice;

   //
   // USB handle to the device
   //
    HANDLE   hUsbDevice;

   //
   // USBD Function table
   //
   LPCUSB_FUNCS   UsbFuncs;

   //
   // Fields from USB_INTERFACE that we need
   //
   UCHAR bInterfaceNumber;
   UCHAR bAlternateSetting;

   USHORT ConfigIndex;

   //
   // Bulk OUT Pipe
   //
   PIPE BulkOut;

   //
   // Bulk IN Pipe
   //
   PIPE BulkIn;

    //
    // completion signal used for endpoint 0
    //
    HANDLE hEP0Event;


   //
   // Signals the device has been closed.
   //
   HANDLE hCloseEvent;

   //
   // FLAGS
   //
   FLAGS Flags;

   //
   // USB Timeouts
   //
   USBTIMEOUTS UsbTimeouts;

} ECAIOUSB_CONTEXT, *PECAIOUSB_CONTEXT;


//
// Is the pointer a valid PECAIOUSB_CONTEXT
//
#define VALID_CONTEXT( p ) \
   ( p && USB_EAU_SIG == p->Sig )


//
// Can the device accept any I/O requests
//
#define ACCEPT_IO( p ) \
   ( VALID_CONTEXT( p ) && \
     p->Flags.Open && \
    !p->Flags.UnloadPending )


//
// USB_DRIVER_SETTINGS
//
#define DRIVER_SETTINGS \
            sizeof(USB_DRIVER_SETTINGS),  \
            USB_EAU_VENDOR_ID,   \
            USB_EAU_PRODUCT_ID,   \
            USB_NO_INFO,   \
            USB_NO_INFO,   \
            USB_NO_INFO,   \
            USB_NO_INFO,   \
            USB_NO_INFO,      \
            USB_NO_INFO,   \
            USB_NO_INFO

#define DRIVER_SETTINGS_PRE \
            sizeof(USB_DRIVER_SETTINGS),  \
            USB_EAU_VENDOR_ID,   \
            USB_EAU_PRODUCT_ID_PRE,   \
            USB_NO_INFO,   \
            USB_NO_INFO,   \
            USB_NO_INFO,   \
            USB_NO_INFO,   \
            USB_NO_INFO,      \
            USB_NO_INFO,   \
            USB_NO_INFO


//
// ECAIOUSB.C
//
BOOL
ParseUsbDescriptors(
   USB_HANDLE       hUsbDevice,
   LPCUSB_FUNCS     UsbFuncs,
   LPCUSB_INTERFACE CurInterface,
   LPUSHORT         ConfigIndex,
   LPCUSB_INTERFACE pUsbInterface
   );

BOOL
SetUsbInterface(
   PECAIOUSB_CONTEXT  pContext,
   LPCUSB_INTERFACE UsbInterface
   );

DWORD
GetDeviceId(
   PECAIOUSB_CONTEXT pContext,
   PBYTE           pBuf,
   DWORD           dwLen
   );

USHORT
GetPortStatus(
   PECAIOUSB_CONTEXT pContext
   );

BOOL
SoftReset(
   PECAIOUSB_CONTEXT pContext
   );

DWORD
IoErrorHandler(
   PECAIOUSB_CONTEXT pContext,
   USB_PIPE        hPipe,
   UCHAR           bIndex,
   USB_ERROR       dwUsbError
   );


//
// DEBUG
//
#if DEBUG

#define ZONE_ERR            DEBUGZONE(0)
#define ZONE_WARN           DEBUGZONE(1)
#define ZONE_INIT           DEBUGZONE(2)
#define ZONE_TRACE          DEBUGZONE(3)

#define ZONE_EAU_INIT       DEBUGZONE(4)
#define ZONE_EAU_READ       DEBUGZONE(5)
#define ZONE_EAU_WRITE      DEBUGZONE(6)
#define ZONE_EAU_IOCTL      DEBUGZONE(7)

#define ZONE_USB_PARSE      DEBUGZONE(8)
#define ZONE_USB_INIT       DEBUGZONE(9)
#define ZONE_USB_CONTROL    DEBUGZONE(10)
#define ZONE_USB_BULK       DEBUGZONE(11)

#define ZONE_USBCLIENT      DEBUGZONE(15)

//
// these should be removed in the code if you can 'g' past these successfully
//
#define TEST_TRAP() { \
   NKDbgPrintfW( TEXT("%s: Code Coverage Trap in: %s, Line: %d\n"), DRIVER_NAME, TEXT(__FILE__), __LINE__); \
   DebugBreak();  \
}

#define DUMP_USB_DEVICE_DESCRIPTOR( d ) { \
   DEBUGMSG( ZONE_USB_PARSE, (TEXT("USB_DEVICE_DESCRIPTOR:\n"))); \
   DEBUGMSG( ZONE_USB_PARSE, (TEXT("----------------------\n"))); \
   DEBUGMSG( ZONE_USB_PARSE, (TEXT("bLength: 0x%x\n"), d.bLength ));   \
   DEBUGMSG( ZONE_USB_PARSE, (TEXT("bDescriptorType: 0x%x\n"), d.bDescriptorType ));  \
   DEBUGMSG( ZONE_USB_PARSE, (TEXT("bcdUSB: 0x%x\n"), d.bcdUSB ));  \
   DEBUGMSG( ZONE_USB_PARSE, (TEXT("bDeviceClass: 0x%x\n"), d.bDeviceClass ));  \
   DEBUGMSG( ZONE_USB_PARSE, (TEXT("bDeviceSubClass: 0x%x\n"), d.bDeviceSubClass ));  \
   DEBUGMSG( ZONE_USB_PARSE, (TEXT("bDeviceProtocol: 0x%x\n"), d.bDeviceProtocol ));  \
   DEBUGMSG( ZONE_USB_PARSE, (TEXT("bMaxPacketSize0: 0x%x\n"), d.bMaxPacketSize0 ));  \
   DEBUGMSG( ZONE_USB_PARSE, (TEXT("idVendor: 0x%x\n"), d.idVendor )); \
   DEBUGMSG( ZONE_USB_PARSE, (TEXT("idProduct: 0x%x\n"), d.idProduct ));  \
   DEBUGMSG( ZONE_USB_PARSE, (TEXT("bcdDevice: 0x%x\n"), d.bcdDevice ));  \
   DEBUGMSG( ZONE_USB_PARSE, (TEXT("iManufacturer: 0x%x\n"), d.iManufacturer ));   \
   DEBUGMSG( ZONE_USB_PARSE, (TEXT("iProduct: 0x%x\n"), d.iProduct )); \
   DEBUGMSG( ZONE_USB_PARSE, (TEXT("iSerialNumber: 0x%x\n"), d.iSerialNumber ));   \
   DEBUGMSG( ZONE_USB_PARSE, (TEXT("bNumConfigurations: 0x%x\n"), d.bNumConfigurations ));  \
   DEBUGMSG( ZONE_USB_PARSE, (TEXT("\n")));  \
}

#define DUMP_USB_CONFIGURATION_DESCRIPTOR( c ) { \
   DEBUGMSG( ZONE_USB_PARSE, (TEXT("USB_CONFIGURATION_DESCRIPTOR:\n"))); \
   DEBUGMSG( ZONE_USB_PARSE, (TEXT("-----------------------------\n"))); \
   DEBUGMSG( ZONE_USB_PARSE, (TEXT("bLength: 0x%x\n"), c.bLength )); \
   DEBUGMSG( ZONE_USB_PARSE, (TEXT("bDescriptorType: 0x%x\n"), c.bDescriptorType )); \
   DEBUGMSG( ZONE_USB_PARSE, (TEXT("wTotalLength: 0x%x\n"), c.wTotalLength )); \
   DEBUGMSG( ZONE_USB_PARSE, (TEXT("bNumInterfaces: 0x%x\n"), c.bNumInterfaces )); \
   DEBUGMSG( ZONE_USB_PARSE, (TEXT("bConfigurationValue: 0x%x\n"), c.bConfigurationValue )); \
   DEBUGMSG( ZONE_USB_PARSE, (TEXT("iConfiguration: 0x%x\n"), c.iConfiguration )); \
   DEBUGMSG( ZONE_USB_PARSE, (TEXT("bmAttributes: 0x%x\n"), c.bmAttributes )); \
   DEBUGMSG( ZONE_USB_PARSE, (TEXT("MaxPower: 0x%x\n"), c.MaxPower )); \
   DEBUGMSG( ZONE_USB_PARSE, (TEXT("\n"))); \
}

#define DUMP_USB_INTERFACE_DESCRIPTOR( i, _index ) { \
   DEBUGMSG( ZONE_USB_PARSE, (TEXT("USB_INTERFACE_DESCRIPTOR[%d]:\n"), _index )); \
   DEBUGMSG( ZONE_USB_PARSE, (TEXT("-------------------------\n"))); \
   DEBUGMSG( ZONE_USB_PARSE, (TEXT("bLength: 0x%x\n"), i.bLength )); \
   DEBUGMSG( ZONE_USB_PARSE, (TEXT("bDescriptorType: 0x%x\n"), i.bDescriptorType )); \
   DEBUGMSG( ZONE_USB_PARSE, (TEXT("bInterfaceNumber: 0x%x\n"), i.bInterfaceNumber )); \
   DEBUGMSG( ZONE_USB_PARSE, (TEXT("bAlternateSetting: 0x%x\n"), i.bAlternateSetting )); \
   DEBUGMSG( ZONE_USB_PARSE, (TEXT("bNumEndpoints: 0x%x\n"), i.bNumEndpoints )); \
   DEBUGMSG( ZONE_USB_PARSE, (TEXT("bInterfaceClass: 0x%x\n"), i.bInterfaceClass )); \
   DEBUGMSG( ZONE_USB_PARSE, (TEXT("bInterfaceSubClass: 0x%x\n"), i.bInterfaceSubClass )); \
   DEBUGMSG( ZONE_USB_PARSE, (TEXT("bInterfaceProtocol: 0x%x\n"), i.bInterfaceProtocol )); \
   DEBUGMSG( ZONE_USB_PARSE, (TEXT("iInterface: 0x%x\n"), i.iInterface )); \
   DEBUGMSG( ZONE_USB_PARSE, (TEXT("\n"))); \
}

#define DUMP_USB_ENDPOINT_DESCRIPTOR( e ) { \
   DEBUGMSG( ZONE_USB_PARSE, (TEXT("USB_ENDPOINT_DESCRIPTOR:\n"))); \
   DEBUGMSG( ZONE_USB_PARSE, (TEXT("-----------------------------\n"))); \
   DEBUGMSG( ZONE_USB_PARSE, (TEXT("bLength: 0x%x\n"), e.bLength )); \
   DEBUGMSG( ZONE_USB_PARSE, (TEXT("bDescriptorType: 0x%x\n"), e.bDescriptorType )); \
   DEBUGMSG( ZONE_USB_PARSE, (TEXT("bEndpointAddress: 0x%x\n"), e.bEndpointAddress )); \
   DEBUGMSG( ZONE_USB_PARSE, (TEXT("bmAttributes: 0x%x\n"), e.bmAttributes )); \
   DEBUGMSG( ZONE_USB_PARSE, (TEXT("wMaxPacketSize: 0x%x\n"), e.wMaxPacketSize )); \
   DEBUGMSG( ZONE_USB_PARSE, (TEXT("bInterval: 0x%x\n"), e.bInterval ));\
   DEBUGMSG( ZONE_USB_PARSE, (TEXT("\n"))); \
}

#else
#define TEST_TRAP()
#define DUMP_USB_DEVICE_DESCRIPTOR( p )
#define DUMP_USB_CONFIGURATION_DESCRIPTOR( c )
#define DUMP_USB_INTERFACE_DESCRIPTOR( i, _index )
#define DUMP_USB_ENDPOINT_DESCRIPTOR( e )
#endif // DEBUG


#endif // _ECAIOUSB_

// EOF